/****************************************************************************************************************
   Program: 240_create_nhc24_rural.sas

   Purpose: This program uses the geocoded, most recent month of publicly available Nursing Home Compare data
            (the output file from the 237 program) to assign facilities to Rural-Urban Commuting Area (RUCA)
            codes and to construct a binary rural variable based on the RUCA values.  It uses the publicly
            available 2010 census tract to RUCA code crosswalk available from the U.S. Department of Agriculture
            (https://www.ers.usda.gov/data-products/rural-urban-commuting-area-codes/documentation). In this
            program, the code also crosswalks the SSA county codes from Nursing Home Compare to FIPS county
            codes using the publicly available crosswalk from the National Bureau of Economic Research
            (https://www.nber.org/research/data/ssa-federal-information-processing-series-fips-state-
            and-county-crosswalk).

   Input files: ruca_xwalk.sas7bdat
                nhc_2024.sas7bdat
                nhc_2024_tracts.sas7bdat
                ssa_fips_state_county2020.sas7bdat

   Output files: nhc_2024_w_rural.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

libname in "<input directory>" access=readonly;
libname out "<output directory>";

%let in_nhc = in.nhc_2024;
%let in_tract = in.nhc_2024_tracts;
%let in_ruca = in.ruca_xwalk;
%let in_fips = in.ssa_fips_state_county2020;
%let outfile = out.nhc_2024_w_rural;

data ruca_xwalk(rename=(tract_fips_cd = nhc24_tract_fips_cd
                        primary_ruca_cd = nhc24_primary_ruca
                        secondary_ruca_cd = nhc24_secondary_ruca));
   set &in_ruca.(keep=tract_fips_cd primary_ruca_cd secondary_ruca_cd);

   if 1 <= primary_ruca_cd <= 6 then nhc24_rural = 0;
   else if 7 <= primary_ruca_cd <= 10 then nhc24_rural = 1;
   else nhc24_rural = .;

run;

data nhc_full;
   set &in_nhc.;
run;

proc sort data=nhc_full;
   by nhc24_prov_state nhc24_prov_cnty_cd;
run;

data ssa_to_fips(drop=ssacd);
   set &in_fips.(keep=state ssacd fipscounty
                 rename=(state = nhc24_prov_state
                         fipscounty = nhc24_county_fips_cd));
   length nhc24_prov_cnty_cd $ 3;
   nhc24_prov_cnty_cd = substr(ssacd,3,3);
run;

proc sort data=ssa_to_fips;
   by nhc24_prov_state nhc24_prov_cnty_cd;
run;

data nhc_full(drop=has_fips_cnty)
     ssa_to_fips_qa(keep=has_fips_cnty);
   merge nhc_full(in=a) ssa_to_fips(in=b);
   by nhc24_prov_state nhc24_prov_cnty_cd;
   if a;
   has_fips_cnty = b;
run;

proc sort data=nhc_full;
   by nhc24_ccn;
run;

data nhc_tract;
   set &in_tract.(rename=(geoid = nhc24_tract_fips_cd
                          match = nhc24_tract_match));
run;

proc sort data=nhc_tract;
   by nhc24_tract_fips_cd;
run;

data nhc_tract_ruca;
   merge nhc_tract(in=a) ruca_xwalk(in=b);
   by nhc24_tract_fips_cd;
   if a;
run;

proc sort data=nhc_tract_ruca;
   by nhc24_ccn;
run;

data &outfile.;
   merge nhc_full nhc_tract_ruca;
   by nhc24_ccn;
run;