/****************************************************************************************************************
   Program Name: 410_avg_daily_census.sas

   Description: This program calculates the average daily census overall and for people living with dementia and  
                by race and ethnicity among all facilities in the 2021 MDS. It also merges the MDS facility-level 
                file with average daily census data to the file with LTC Focus and Nursing Home Compare data and flags facilities that have  
                changed CCNs between 2021 and 2024.  
				
   Input files: _400_mds_2020.sas7bdat
				_400_mds_2021.sas7bdat
                all_nh_facilities.sas7bdat

   Output files: _410_avg_daily_census.sas7bdat
****************************************************************************************************************/
options validvarname=upcase mergenoby=error compress=yes;
options mprint symbolgen obs=max nocenter ls=154 ps=58 minoperator;

libname in "<input directory>" access = readonly;
libname out "<output directory>";
libname ccn "<NH file directory>" access = readonly;

%let indata = in._400_mds;
%let outdata = out._410_avg_daily_census;

/* Main Title */
title "Create Average Daily Census 2021";

/* Stack 2020 and 2021 processed MDS data, sort */
data mds_2020_2021;
	set &indata._2021 &indata._2020;
run;

proc sort data = mds_2020_2021;
	by a0100b_cms_crtfctn_num bene_id descending trgt_dt;
run;

/* Set the current and prior year MDS assessments together and prepare to 
   calculate the average daily census by identifying if/when a beneficiary
   was discharged from a facility */
data mds_w_dschrg_2020_2021;
	set mds_2020_2021;
	by a0100b_cms_crtfctn_num bene_id;
	retain prev_trgt_dt;

	if first.bene_id then do;
		subsequent_record_wi_150_days = .;
	end;
	else do;
		if intck('day', trgt_dt, prev_trgt_dt) <= 150 then subsequent_record_wi_150_days = 1;
		else subsequent_record_wi_150_days = 0;
	end;
	prev_trgt_dt = trgt_dt;

	if a0310f_entry_dschrg_cd in ("10", "11") then discharged = 1;
	else if a0310f_entry_dschrg_cd not in ("10", "11", "12") and trgt_dt <= mdy(8,3,2021) and subsequent_record_wi_150_days = 0 then discharged = 2;
	else discharged = 0;

	format prev_trgt_dt end_dt date9.;
	if died = 1 or discharged = 1 then end_dt = trgt_dt;
	else if discharged = 2 then end_dt = intnx('day', trgt_dt, 150);

	gone = (died = 1 or discharged in (1, 2));
	missing_bene_id = (missing(bene_id));
	missing_trgt_dt = (missing(trgt_dt));
	missing_end_dt = (missing(end_dt));
	drop prev_trgt_dt;
run;

proc sort data = mds_w_dschrg_2020_2021;
	by a0100b_cms_crtfctn_num bene_id trgt_dt gone discharged;
run;

/* Create a stayid to identify all assessments that were part of the same stay 
   for the same resident in a facility */
data mds_w_stayid_2020_2021;
	set mds_w_dschrg_2020_2021;
	by a0100b_cms_crtfctn_num bene_id;
	retain prev_stayid prev_stayid_seq prev_gone prev_trgt_dt;
	switch_1_to_0 = 0;
	if _N_ = 1 then prev_stayid = 0;
	if first.bene_id then do;
		stayid = prev_stayid + 1;
		stayid_seq = 1;
	end;
	else do;
		if prev_gone = 1 and prev_trgt_dt ne trgt_dt then do;
			stayid = prev_stayid + 1;
			stayid_seq = 1;
			if gone = 0 then switch_1_to_0 = 1;
		end;
		else do;
			stayid = prev_stayid;
			stayid_seq = prev_stayid_seq + 1;
		end;
	end;
	prev_stayid = stayid;
	prev_stayid_seq = stayid_seq;
	prev_gone = gone;
	prev_trgt_dt = trgt_dt;
run;

proc sql;
	create table mds_w_stayid2_2020_2021 as
	select *,
	 	   min(trgt_dt) as min_tgt_dt format = date9.,
	 	   max(end_dt) as max_end_dt format = date9.
	from mds_w_stayid_2020_2021 (drop = prev_:) as a
	group by stayid
	order by a0100b_cms_crtfctn_num, bene_id, trgt_dt;
quit;

data mds_w_stayid2_2020_2021;
	set mds_w_stayid2_2020_2021;
	if not missing(max_end_dt) and max_end_dt < mdy(1,1,2021) then delete;
run;

/* Collapse file to the stayid level */
proc sql;
	create table mds_stayid_clps_2021 (drop = max_stayid_seq) as 
	select *, sum(intck('day', firstday_2021, lastday_2021), 1) as tot_days
	from
	(
	select a.*,
		   b.a1000a_amrcn_indn_ak_ntv_cd,
		   b.a1000b_asn_cd,
		   b.a1000c_afrcn_amrcn_cd,
		   b.a1000d_hspnc_cd,
		   b.a1000e_ntv_hi_pcfc_islndr_cd,
		   b.a1000f_wht_cd,
		   case when min_tgt_dt <= mdy(1,1,2021) then mdy(1,1,2021) else min_tgt_dt end as firstday_2021 format = date9.,
		   case when not missing(max_end_dt) then max_end_dt else mdy(12,31,2021) end as lastday_2021 format = date9.
	from (select a0100b_cms_crtfctn_num,
		   		 bene_id,
		   		 stayid,
				 exclude,
			     min_tgt_dt,
			     max_end_dt,
				 max(stayid_seq) as max_stayid_seq,
			     max(dement) as dement,
			     max(gone) as gone,
			     max(died) as died,
			     max(discharged) as discharged
		  from mds_w_stayid2_2020_2021
		  group by a0100b_cms_crtfctn_num, bene_id, stayid, exclude, min_tgt_dt, max_end_dt) as a
	left join
	(select a0100b_cms_crtfctn_num,
		   	bene_id,
		   	stayid,
		   	stayid_seq,
			a1000a_amrcn_indn_ak_ntv_cd,
			a1000b_asn_cd,
			a1000c_afrcn_amrcn_cd,
			a1000d_hspnc_cd,
			a1000e_ntv_hi_pcfc_islndr_cd,
			a1000f_wht_cd
	 from mds_w_stayid2_2020_2021) as b
	on a.a0100b_cms_crtfctn_num = b.a0100b_cms_crtfctn_num and a.bene_id = b.bene_id and a.stayid = b.stayid and a.max_stayid_seq = b.stayid_seq
	);
quit;

/* Collapse the file to the facility level and calculate average daily census */
proc sql;
	create table mds_census_2021 as
	select a0100b_cms_crtfctn_num,
		   exclude,
		   sum(tot_days) as fac_census_all label = "Total Number of Resident Days in the Year",
		   sum(case when dement = 1 then tot_days else 0 end) as fac_census_all_dement label = "Total Number of Resident Days Among Residents with Dementia in the Year",
		   sum(case when a1000f_wht_cd = "1" then tot_days else 0 end) as fac_census_wht label = "Total Number of Resident Days Among White Residents in the Year",
		   sum(case when a1000f_wht_cd = "1" and dement = 1 then tot_days else 0 end) as fac_census_wht_dement label = "Total Number of Resident Days Among White Residents with Dementia in the Year",
		   count(distinct bene_id) as fac_num_res label = "Number of Unique Patients Served by Facility During the Year",
		   count(distinct case when dement = 1 then bene_id else . end) as fac_num_res_dement label = "Number of Unique Patients with Dementia Served by Facility During the Year"
	from mds_stayid_clps_2021 
	group by a0100b_cms_crtfctn_num, exclude;
quit;

data mds_census_2021;
	set mds_census_2021;
	if missing(a0100b_cms_crtfctn_num) then delete; 
	fac_census_nonwht = sum(fac_census_all, -fac_census_wht);
	fac_census_nonwht_dement = sum(fac_census_all_dement, -fac_census_wht_dement);
	avg_daily_census_all = fac_census_all / 365;
	avg_daily_census_dmt = fac_census_all_dement / 365;
	avg_daily_census_wht = fac_census_wht / 365;
	avg_daily_census_wht_dmt = fac_census_wht_dement / 365;
	avg_daily_census_nonwht = fac_census_nonwht / 365;
	avg_daily_census_nonwht_dmt = fac_census_nonwht_dement / 365;
	if fac_num_res ne 0 then pct_dement_rsdnts = fac_num_res_dement / fac_num_res;
	if avg_daily_census_all ne 0 then pct_dement_days = avg_daily_census_dmt / avg_daily_census_all;
	if avg_daily_census_all ne 0 then pct_nonwht_days = avg_daily_census_nonwht / avg_daily_census_all;
	if avg_daily_census_dmt ne 0 then pct_nonwht_dement_days = avg_daily_census_nonwht_dmt / avg_daily_census_dmt;
	no_dement_days = (fac_census_all_dement = 0);
	mds = 1;
	label fac_census_nonwht = "Total Number of Resident Days Among Non-White Residents in the Year"
		  fac_census_nonwht_dement = "Total Number of Resident Days Among Non-White Residents with Dementia in the Year"
		  avg_daily_census_all = "Average Daily Census in the Facility"
		  avg_daily_census_dmt = "Average Daily Census of Residents with Dementia in the Facility"
		  avg_daily_census_wht = "Average Daily Census of White Residents in the Facility"
		  avg_daily_census_wht_dmt = "Average Daily Census of White Residents with Dementia in the Facility"
		  avg_daily_census_nonwht = "Average Daily Census of Non-White Residents in the Facility"
		  avg_daily_census_nonwht_dmt = "Average Daily Census of Non-White Residents with Dementia in the Facility"
		  pct_dement_rsdnts = "Percent of Residents Who Ever Stays at the NH in the Year with Dementia"
		  pct_dement_days = "Percent of Resident Days with Dementia in the Facility"
		  pct_nonwht_days = "Percent of Resident Days Who Are Non-White in the Facility"
		  pct_nonwht_dement_days = "Percent of Dementia Resident Days Who Are Non-White in the Facility"
		  mds = "Facility in MDS Data"
		  no_dement_days = "CCN Does Not Have Dementia Days";
	;
	format fac_census: fac_num_res: mask.;
run;

/* Merge the MDS to the file with LTC Focus and Nursing Home Compare data */
proc sort data=mds_census_2021(rename=(a0100b_cms_crtfctn_num = ccn)); by ccn; run;

proc sort data=ccn.all_nh_facilities out=ccn_file; by ccn; run;

data mds_ccn;
merge mds_census_2021 (in=_inmds) ccn_file (in=_inccn);
by ccn;
inmds = _inmds;
inccn = _inccn;
has_ltcf21 = (fac_file in (1,3,4,5));
has_nhc21 = (fac_file in (1,2,3,6));
has_nhc24 = (fac_file in (1,2,4,7));
run;

/* Look for CCNs that changed ownership between 2021 and 2024*/

/* Identify facilities in Nursing Home Compare 2024 data that did not have a CCN 
   in the MDS and create a variable concatenating the 2024 facility address and zip 
   code for those facilities */
data no_nhc24;
set mds_ccn;
where missing(mds) and fac_file in (1,2,4,7);
addr_zip = catx('-', nhc24_prov_addr, nhc24_prov_zip);
run;

/* Retain a small set of variables, mostly from 2024 Nursing Home Compare, for the facilities in Nursing Home Compare 2024 
   and not in MDS */
data no_nhc24;
set no_nhc24;
ccn24 = ccn;
fac_file24 = fac_file;
keep ccn24 fac_file24 addr_zip nhc24_prov_addr nhc24_prov_zip nhc24_prov_name
	 nhc24_prov_type nhc24_resides_in_hospital nhc24_cont_care_rtrmnt_cmnty
	 nhc24_avg_nmbr_rsdnts_per_day_fn nhc24_nmbr_crtfd_beds nhc24_ownership_type
	 nhc24_changed_owner_lst12mnths
	 ;
run;

/* Put all of the concatenated address-zip codes for facilities in Nursing Home Compare 2024 and that
   were not found in MDS into a macro variable that will be used to flag facilities in the MDS whose 2021 
   Nursing Home Compare address-zip data were the same */

proc sql noprint;
select quote(trim(addr_zip)) into :addrzip24 separated by ','
from no_nhc24;
quit;

/* Identify records in the MDS with a CCN in Nursing Home Compare 2021
   and flag the 2021 facilities with the same address and zip code as the 
   2024 Nursing Home Compare CCNs that were not in MDS */
data nhc21;
set mds_ccn;
where mds = 1 and fac_file in (1,2,3,6);
addr_zip = catx('-', nhc21_prov_addr, nhc21_prov_zip);
if trim(addr_zip) in (&addrzip24.) then flag = 1; 
run;

/* Limit to the set of CCNs that had the same address and zip code in 2021 and 
   2024 and keep 2021 Nursing Home Compare variables */
data matches;
set nhc21;
where flag = 1;
length ccn21 $12.;
ccn21 = ccn;
fac_file21 = fac_file;
keep ccn21 fac_file21 addr_zip nhc21_prov_addr nhc21_prov_zip nhc21_prov_name
	 nhc21_prov_type nhc21_resides_in_hospital nhc21_cont_care_rtrmnt_cmnty
	 nhc21_avg_nmbr_rsdnts_per_day_fn nhc21_nmbr_crtfd_beds nhc21_ownership_type
	 nhc21_changed_owner_lst12mnths
	 ;
run;

proc sort data=matches; by addr_zip; run;
proc sort data=no_nhc24; by addr_zip; run;

/* Merge the CCNs in the MDS file (and not in the 2024 Nursing Home Compare) 
   with the CCNS in the 2024 Nursing Home Compare (and not in MDS) by address-zip 
   and keep CCNs that matched. This file contains CCNs that changed ownership between 
   2021 and 2024 and contains their 2021 and 2024 Nursing Home Compare variables for comparison. 
   Reset the fac_file indicators to indicate the CCN was found in 2021 and 2024 */
data ownership_x_flg;
merge matches (in=a) no_nhc24 (in=b);
by addr_zip;
if a;
/* Reset fac_file for these records*/
if fac_file21 = 2 then do;
	if fac_file24 = 7 then fac_file_new = 2;
end;
else if fac_file21 = 3 then do;
	if fac_file24 = 2 then fac_file_new = 1;
	else if fac_file24 = 7 then fac_file_new = 1;
end;
else if fac_file21 = 6 then do;
	if fac_file24 = 2 then fac_file_new = 2;
	else if fac_file24 = 7 then fac_file_new = 2;
end;
ccn = ccn21;
run;

/* Exclude 3 CCNs from the set of changed ownership CCNs based on 
   information from the facility websites */
data new_ccns;
set ownership_x_flg;
keep ccn ccn24;
if ccn in ('445536', '16E754', '165788') then delete;
run;

proc sort data=mds_census_2021; by ccn; run;
proc sort data=new_ccns; by ccn; run;

/* Reset the CCN on the MDS file to the 2024 Nursing Home Compare CCN
   for the facilities that changed ownership */
data mds_census_2021_fixed;
merge mds_census_2021 (in=_a) new_ccns (in=_b);
by ccn;
if _b then ccn = ccn24;
run;

proc sort data=mds_census_2021_fixed; by ccn; run;
proc sort data=ccn_file; by ccn; run;

/* Re-merge the LTC Focus and Nursing Home Compare file onto the MDS
   with the changed ownership CCNs updated */
data mds_ccn_fixed;
merge mds_census_2021_fixed (in=a) ccn_file (in=b);
by ccn;
has_ltcf21 = (fac_file in (1,3,4,5));
has_nhc21 = (fac_file in (1,2,3,6));
has_nhc24 = (fac_file in (1,2,4,7));
run;

/* Flag facilities that have the same name and zip code or address and zip code, 
   but different CCNs, indicating that they are likely located on the same campus. 
   Also create an ID variable so the sampling team can track these facilities */
data in_nhc24;
set mds_ccn;
where mds = 1 and has_nhc24 = 1;
run;

/* Finding facilities with the same address and zip code */
proc sql;
create table same_addr_zip as
select nhc24_prov_addr, nhc24_prov_zip, count(distinct ccn) as num_ccns
from in_nhc24
group by nhc24_prov_addr, nhc24_prov_zip;
quit;

data same_addr_zip;
set same_addr_zip;
if missing(nhc24_prov_addr) then delete;
same_fac_addr = 0;
same_fac_id_addr = 0;
if num_ccns > 1 then do;
	same_fac_addr = 1;
	same_fac_id_addr = _N_;
end;
run;

/* Finding facilities with the same name and zip code*/
proc sql;
create table same_name_zip as
select nhc24_prov_name, nhc24_prov_zip, count(distinct ccn) as num_ccns
from in_nhc24
group by nhc24_prov_name, nhc24_prov_zip;
quit;

data same_name_zip;
set same_name_zip;
if missing(nhc24_prov_name) then delete;
same_fac_name = 0;
same_fac_id_name = 0;
if num_ccns > 1 then do;
	same_fac_name = 1;
	same_fac_id_name = _N_;
end;
run;

proc sort data=same_addr_zip(drop=num_ccns); by nhc24_prov_addr nhc24_prov_zip; run;
proc sort data=same_name_zip(drop=num_ccns); by nhc24_prov_name nhc24_prov_zip; run;

proc sql;
create table mds_ccn_4 as
select a.*, b.same_fac_addr, b.same_fac_id_addr,
		c.same_fac_name, c.same_fac_id_name
from mds_ccn as a

left join same_addr_zip as b
on a.nhc24_prov_addr = b.nhc24_prov_addr
and a.nhc24_prov_zip = b.nhc24_prov_zip

left join same_name_zip as c
on a.nhc24_prov_name = c.nhc24_prov_name
and a.nhc24_prov_zip = c.nhc24_prov_zip;
quit;

data mds_ccn_5;
set mds_ccn_4;
if same_fac_addr = 1 or same_fac_name = 1 then same_fac = 1;
if same_fac_addr = 0 and same_fac_name = 0 then same_fac = 0;

if same_fac_addr = 1 then same_fac_id = same_fac_id_addr;
else if same_fac_name = 1 then same_fac_id = same_fac_id_name;

run;

data mds_ccn_6;
set mds_ccn_5;
where not missing(mds);
run;

data mds_ccn_7;
set mds_ccn_6;
if has_nhc24 = 0 and exclude ne 1 then exclude = 2;
run;

/*Create final file for this program*/
data &outdata.;
set mds_ccn_7;
where exclude ne 2;
drop inccn inmds same_fac_addr same_fac_id_addr same_fac_name same_fac_id_name
	 in_nhc21 in_nhc24 in_ltcf fac_file ltc_prov_name 
	 ltc_prov_addr ltc_prov_zip ltc_prov_city ltc_prov_state ltc_prov_cnty_cd
	 nhc21_prov_name nhc21_prov_addr nhc21_prov_city nhc21_prov_state
	 nhc21_prov_zip nhc21_prov_phone nhc21_prov_cnty_cd nhc21_prov_cnty_name 
	 nhc21_ownership_type nhc21_nmbr_crtfd_beds nhc21_prov_type
	 nhc21_resides_in_hospital nhc21_legal_bus_name nhc21_date_first_approved 
	 nhc21_cont_care_rtrmnt_cmnty nhc21_hlth_insp_gt_2yrs_ago
	 nhc21_changed_owner_lst12mnths
	 ;
run;