/****************************************************************************************************************
   Program: 440_nh_sampmle_frame.sas

   Purpose: Prepare the final Nursing Home Sample Frame file:
             1) Move the variables that will be most important for sampling to be the first variables listed on 
                the file
             2) Remove variables that are not needed (primarily address-related variables from Nursing Home 
                Compare 2021 and LTC Focus)
             3) Remove duplicate records for two facilities (all variables contained the same values across the 
                two records for the facility, including name and address, except for a few staffing-related variables)
             4) Remove facilities located in U.S. territories
             5) Create the final set of informative missingness indicators:
                 .M = "Missing" if a facility is missing information for a particular variable
                 .E = "Eleven" if a cell count is smaller than 11 (and was formated as "<11" in prior program)
                 .S = "Suppressed" if a cell was suppressed (formatted with an asterisk in prior program) 

   Input files: _430_avg_dly_census_w_all_vars.sas7bdat
                all_nh_facilities.sas7bdat

   Output files: NH_frame_{yyyymmdd}.sas7bdat
                 NH_frame_{yyyymmdd}.xlsx
****************************************************************************************************************/
options nocenter mprint obs=max compress=yes;

libname in "<input directory>" access=readonly;
libname out "<output directory>";

%let dt=%sysfunc(today(),yymmddn8.);

%let infile = in._430_avg_dly_census_w_all_vars;;
%let outfile = out.NH_frame_&dt.;
%let xlsfile = "<output directory>\NH_frame_&dt..xlsx";

***********************************************************************
*-- Setting the order of variables so that key sample frame variables 
    come first
*********************************************************************** ;

%let var_order = 
CCN
SAME_FAC
SAME_FAC_ID
AVG_DAILY_CENSUS_ALL
AVG_DAILY_CENSUS_DMT
LTC_ALZUNIT_NMRC
LTC_PAYMCAID
NHC24_NMBR_CRTFD_BEDS
NHC24_PRIMARY_RUCA
NHC24_RURAL
MDS
HAS_NHC24
HAS_LTCF21
HAS_NHC21
;

***********************************************************************
*-- Variables to drop
*********************************************************************** ;

%let drop_vars = 
IN_NHC21 
IN_NHC24
IN_LTCF
FAC_FILE  
LTC_prov_name 
LTC_prov_addr
LTC_PROV_CITY 
LTC_prov_zip 
LTC_ALZUNIT
LTC_PROFIT
LTC_PROV_STATE 
LTC_PROV_CNTY_CD 
LTC_MSNG_ORIG_CCN
nhc21_legal_bus_name
nhc21_prov_name 
nhc21_prov_addr  
nhc21_prov_city  
nhc21_prov_state  
nhc21_prov_zip  
nhc21_prov_phone  
nhc21_prov_cnty_cd  
nhc21_prov_cnty_name  
nhc21_ownership_type  
nhc21_nmbr_crtfd_beds  
nhc21_prov_type  
nhc21_resides_in_hospital  
nhc21_cont_care_rtrmnt_cmnty  
nhc21_hlth_insp_gt_2yrs_ago   
nhc21_changed_owner_lst12mnths 
nhc21_month
nhc21_date_first_approved
;

***********************************************************************
*-- Read in the NH file and drop variables we do not need 
*********************************************************************** ;

data nh_sample_frame;
  set &infile.;
  drop &drop_vars.;
run;

***********************************************************************
*-- Remove the duplicate CCNs from the file
*********************************************************************** ;

data nh_sample_frame_deduped;
  set nh_sample_frame;
  by ccn;
  if first.ccn;
run;

data dupes;
  set nh_sample_frame;
  by ccn;
  if first.ccn and last.ccn then delete;
run;

***********************************************************************
*-- Remove nursing homes in US territories
*********************************************************************** ;

data nh_sample_frame_no_terr;
  set nh_sample_frame_deduped;
  if nhc24_prov_state in ('PR' 'GU') then delete;
run;

***********************************************************************
*-- Reset variables with missing data to informative, numeric missing
*********************************************************************** ;

data nh_sample_frame_upd;
  set nh_sample_frame_no_terr;

  %macro reset_miss(var=);
  if missing(&var.) then &var._N = .M;
  else if &var. = "*" then &var._N = .S;
  else if &var. = "<11" then &var._N = .E; 
  else &var._N = input(&var., 8.);

  drop &var.;
  rename &var._N = &var.;
  %mend reset_miss;

  %reset_miss(var=AVG_DAILY_CENSUS_ALL);
  %reset_miss(var=AVG_DAILY_CENSUS_DMT);       
  %reset_miss(var=FAC_CENSUS_ALL);             
  %reset_miss(var=FAC_CENSUS_ALL_DEMENT);      
  %reset_miss(var=FAC_CENSUS_WHT);             
  %reset_miss(var=FAC_CENSUS_WHT_DEMENT);      
  %reset_miss(var=FAC_NUM_RES);                
  %reset_miss(var=FAC_NUM_RES_DEMENT);         
  %reset_miss(var=FAC_CENSUS_NONWHT);          
  %reset_miss(var=FAC_CENSUS_NONWHT_DEMENT);   
  %reset_miss(var=AVG_DAILY_CENSUS_WHT);       
  %reset_miss(var=AVG_DAILY_CENSUS_WHT_DMT);   
  %reset_miss(var=AVG_DAILY_CENSUS_NONWHT);    
  %reset_miss(var=AVG_DAILY_CENSUS_NONWHT_DMT);
  %reset_miss(var=PCT_DEMENT_RSDNTS);          
  %reset_miss(var=PCT_DEMENT_DAYS);            
  %reset_miss(var=PCT_NONWHT_DAYS);            
  %reset_miss(var=PCT_NONWHT_DEMENT_DAYS);     
run;

***********************************************************************
*-- Move the key sample frame variables to the front of the file
*********************************************************************** ;

data &outfile.;
  retain &var_order.;
  set nh_sample_frame_upd;
run;

***********************************************************************
*-- Output to Excel
*********************************************************************** ;

proc export data=&outfile.
			outfile=&xlsfile.
			dbms=xlsx
			label
			replace;
run;